#!/usr/bin/env node
/*----------------------------------------------
 * Generated by rollup. Written by John Schmidt.
 * Rally Tools CLI v1.6.2
 *--------------------------------------------*/
const importLazy = require("import-lazy")(require);

'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var os = require('os');
var fs = require('fs');
var fs__default = _interopDefault(fs);
var chalk$1 = _interopDefault(require('chalk'));
var child_process = require('child_process');
var perf_hooks = require('perf_hooks');
var path = require('path');
var path__default = _interopDefault(path);
var argparse = _interopDefault(require('minimist'));

function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) {
  var desc = {};
  Object['ke' + 'ys'](descriptor).forEach(function (key) {
    desc[key] = descriptor[key];
  });
  desc.enumerable = !!desc.enumerable;
  desc.configurable = !!desc.configurable;

  if ('value' in desc || desc.initializer) {
    desc.writable = true;
  }

  desc = decorators.slice().reverse().reduce(function (desc, decorator) {
    return decorator(target, property, desc) || desc;
  }, desc);

  if (context && desc.initializer !== void 0) {
    desc.value = desc.initializer ? desc.initializer.call(context) : void 0;
    desc.initializer = undefined;
  }

  if (desc.initializer === void 0) {
    Object['define' + 'Property'](target, property, desc);
    desc = null;
  }

  return desc;
}

let configFile = os.homedir() + "/.rallyconfig";
let configObject = {
  hasConfig: true
};

try {
  let json = fs.readFileSync(configFile);
  configObject = JSON.parse(json);
} catch (e) {
  if (e.code == "ENOENT") {
    configObject.hasConfig = false; //ok, they should probably make a config
  } else {
    throw e;
  }
}

const rp = importLazy("request-promise");
global.chalk = chalk$1;

global.log = text => console.log(text);

global.write = text => process.stdout.write(text);

global.errorLog = text => log(chalk$1.red(text));

class lib {
  //This function takes 2 required arguemnts:
  // env: the enviornment you wish to use
  // and either:
  //  'path', the short path to the resource. ex '/presets/'
  //  'path_full', the full path to the resource like 'https://discovery-dev.sdvi.com/presets'
  //
  // If the method is anything but GET, either payload or body should be set.
  // payload should be a javascript object to be turned into json as the request body
  // body should be a string that is passed as the body. for example: the python code of a preset.
  //
  // qs are the querystring parameters, in a key: value object.
  // {filter: "name=test name"} becomes something like 'filter=name=test+name'
  //
  // headers are the headers of the request. "Content-Type" is already set if
  //   payload is given as a parameter
  //
  // fullResponse should be true if you want to receive the request object,
  //  not just the returned data.
  static async makeAPIRequest({
    env,
    path: path$$1,
    path_full,
    payload,
    body,
    method = "GET",
    qs,
    headers = {},
    fullResponse = false
  }) {
    var _configObject$api;

    //Keys are defined in enviornment variables
    let config = configObject === null || configObject === void 0 ? void 0 : (_configObject$api = configObject.api) === null || _configObject$api === void 0 ? void 0 : _configObject$api[env];

    if (!config) {
      throw new UnconfiguredEnvError(env);
    }

    if (method !== "GET" && !configObject.dangerModify) {
      if (env === "UAT" && configObject.restrictUAT || env === "PROD") {
        throw new ProtectedEnvError(env);
      }
    }

    let rally_api_key = config.key;
    let rally_api = config.url;
    path$$1 = path_full || rally_api + path$$1;

    if (payload) {
      body = JSON.stringify(payload);
    }

    if (global.logAPI) {
      log(chalk$1`${method} @ ${path$$1}`);

      if (qs) {
        log(qs);
      }
    }

    if (payload) {
      headers["Content-Type"] = "application/vnd.api+json";
    }

    let requestOptions = {
      method,
      body,
      qs,
      uri: path$$1,
      auth: {
        bearer: rally_api_key
      },
      headers: {
        //SDVI ignores this header sometimes.
        Accept: "application/vnd.api+json",
        ...headers
      },
      simple: false,
      resolveWithFullResponse: true
    };
    let response = await rp(requestOptions); //Throw an error for any 5xx or 4xx

    if (!fullResponse && ![200, 201, 204].includes(response.statusCode)) {
      throw new APIError(response, requestOptions, body);
    }

    let contentType = response.headers["content-type"];
    let isJSONResponse = contentType === "application/vnd.api+json" || contentType === "application/json";

    if (fullResponse) {
      return response;
    } else if (isJSONResponse) {
      try {
        return JSON.parse(response.body);
      } catch (e) {
        log(response.body);
        throw new AbortError("Body is not valid json: ");
      }
    } else {
      return response.body;
    }
  } //Index a json endpoint that returns a {links} field.
  //This function returns the merged data objects as an array
  //


  static async indexPath(env, path$$1) {
    let all = [];
    let json = await this.makeAPIRequest(typeof env === "string" ? {
      env,
      path: path$$1
    } : env);
    let [numPages, pageSize] = this.numPages(json.links.last); //log(`num pages: ${numPages} * ${pageSize}`);

    all = [...json.data];

    while (json.links.next) {
      json = await this.makeAPIRequest({
        env,
        path_full: json.links.next
      });
      all = [...all, ...json.data];
    }

    return all;
  } //Returns number of pages and pagination size


  static numPages(str) {
    return /page=(\d+)p(\d+)/.exec(str).slice(1);
  } //Index a json endpoint that returns a {links} field.
  //
  //This function is faster than indexPath because it can guess the pages it
  //needs to retreive so that it can request all assets at once.
  //
  //This function assumes that the content from the inital request is the
  //first page, so starting on another page may cause issues. Consider
  //indexPath for that.


  static async indexPathFast(env, path$$1) {
    let all = [];
    let json = await this.makeAPIRequest(typeof env === "string" ? {
      env,
      path: path$$1
    } : env);
    let baselink = json.links.first;

    const linkToPage = page => baselink.replace("page=1p", `page=${page}p`);

    let [numPages, pageSize] = this.numPages(json.links.last); //log(`num pages: ${numPages} * ${pageSize}`);
    //Construct an array of all the requests that are done simultanously.
    //Assume that the content from the inital request is the first page.

    let promises = [Promise.resolve(json)];

    for (let i = 2; i <= numPages; i++) {
      let req = this.makeAPIRequest({
        env,
        path_full: linkToPage(i)
      });
      promises.push(req);
    }

    for (let promise of promises) {
      all = [...all, ...(await promise).data];
    }

    return all;
  }

  static isLocalEnv(env) {
    return !env || env === "LOCAL" || env === "LOC";
  }

}
class AbortError extends Error {
  constructor(message) {
    super(message);
    Error.captureStackTrace(this, this.constructor);
    this.name = "AbortError";
  }

}
class APIError extends Error {
  constructor(response, opts, body) {
    super(chalk$1`
{reset Request returned} {yellow ${response.statusCode}}{
{green ${JSON.stringify(opts, null, 4)}}
{green ${body}}
{reset ${response.body}}
        `);
    Error.captureStackTrace(this, this.constructor);
    this.name = "ApiError";
  }

}
class UnconfiguredEnvError extends AbortError {
  constructor(env) {
    super("Unconfigured enviornment: " + env);
    this.name = "Unconfigured Env Error";
  }

}
class ProtectedEnvError extends AbortError {
  constructor(env) {
    super("Protected enviornment: " + env);
    this.name = "Protected Env Error";
  }

}
class Collection {
  constructor(arr) {
    this.arr = arr;
  }

  [Symbol.iterator]() {
    return this.arr[Symbol.iterator]();
  }

  findById(id) {
    return this.arr.find(x => x.id == id);
  }

  findByName(name) {
    return this.arr.find(x => x.name == name);
  }

  findByNameContains(name) {
    return this.arr.find(x => x.name.includes(name));
  }

  log() {
    for (let d of this) log(d.chalkPrint(true));
  }

  get length() {
    return this.arr.length;
  }

}
class RallyBase {
  constructor() {}

  resolveApply(datum, dataObj, direction) {
    let obj;

    if (direction == "generic") {
      obj = datum.findById(dataObj.id);

      if (obj) {
        dataObj.name = obj.name;
      }
    } else if (direction == "specific") {
      obj = datum.findByName(dataObj.name);

      if (obj) {
        dataObj.id = obj.id;
      }
    }

    return obj;
  }

  resolveField(datum, name, isArray = false, direction = "generic") {
    let field = this.relationships[name];
    if (!(field === null || field === void 0 ? void 0 : field.data)) return;

    if (isArray) {
      return field.data.map(o => this.resolveApply(datum, o, direction));
    } else {
      return this.resolveApply(datum, field.data, direction);
    }
  }

  cleanup() {
    for (let [key, val] of Object.entries(this.relationships)) {
      if (val.data) {
        if (val.data.id) {
          delete val.data.id;
        } else if (val.data[0]) {
          for (let x of val.data) delete x.id;
        }
      }

      delete val.links;
    }

    delete this.relationships.organization;
    delete this.data.id;
    delete this.data.links;
  }

}

//these are the help entries for each command
let helpEntries = {};

let helpEntry = name => helpEntries[name] ? helpEntries[name] : helpEntries[name] = {
  name
}; //short description


function helpText(text) {
  return function (func, name) {
    helpEntry(name).text = text;
    return func;
  };
} //flag type argument like -f or --file

function arg(long, short, desc) {
  return function (func, name) {
    let args = helpEntry(name).args = helpEntry(name).args || [];
    args.unshift({
      long,
      short,
      desc
    });
    return func;
  };
} //normal argument

function param(param, desc) {
  return function (func, name) {
    let params = helpEntry(name).params = helpEntry(name).params || [];
    params.unshift({
      param,
      desc
    });
    return func;
  };
} //usage string

function usage(usage) {
  return function (func, name) {
    usage = usage.replace(/[\[<](\w+)[\]>]/g, chalk`[{blue $1}]`);
    helpEntry(name).usage = usage;
    return func;
  };
}

function findValueInCache(args, cache) {
  for (let [argsKey, value] of cache) {
    if (args.length !== argsKey.length) continue;

    for (let i in argsKey) {
      if (args[i] === argsKey[i]) {
        return {
          found: true,
          value
        };
      }
    }
  }

  return {
    found: false
  };
} //This decorator takes a function and returns a function that remembers the
//  value returned by given arguments


function cached(target, key, desc) {
  let oldFunc = desc.value;
  let cachedValues = [];

  function newFunc(...args) {
    let {
      found,
      value
    } = findValueInCache(args, cachedValues);

    if (!found) {
      //Call the old function to find the value, then store it in the cache
      value = oldFunc(...args);
      cachedValues.push([args, value]);
    }

    return value;
  }

  newFunc.clearCache = function () {
    cachedValues = [];
  };

  newFunc.cachePush = function (args, value) {
    cachedValues.push([args, value]);
  };

  newFunc.remove = function (...args) {
    let {
      found,
      value
    } = findValueInCache(args, cachedValues);

    if (found) {
      cachedValues = cachedValues.filter(([arg, item]) => item !== value);
    }
  };

  return { ...desc,
    value: newFunc
  };
} //Access a deep property of an object: if path is ["a", "b", "c"], then this
//function retuns obj.a.b.c

function deepAccess(obj, path$$1) {
  let o = obj;

  for (let key of path$$1) {
    if (!o) return [];
    o = o[key];
  }

  return o;
} //This takes a class as the first argument, then adds a getter/setter pair that
//corresponds to an object in this.data


function defineAssoc(classname, shortname, path$$1) {
  path$$1 = path$$1.split(".");
  let lastKey = path$$1.pop();
  Object.defineProperty(classname.prototype, shortname, {
    get() {
      return deepAccess(this.data, path$$1)[lastKey];
    },

    set(val) {
      deepAccess(this.data, path$$1)[lastKey] = val;
    }

  });
}

function spawn(options, ...args) {
  if (typeof options !== "object") {
    args.unshift(options);
    options = {};
  } //todo options


  return new Promise((resolve, reject) => {
    let start = perf_hooks.performance.now();
    let stdout = "";
    let stderr = "";
    let cp = child_process.spawn(...args);
    if (cp.stdout) cp.stdout.on("data", chunk => {
      stdout += chunk;
      write(chunk);
    });
    if (cp.stderr) cp.stderr.on("data", chunk => {
      stderr += chunk;
      write(chunk);
    });
    cp.on("error", reject);
    cp.on("close", code => {
      let end = perf_hooks.performance.now();
      let time = end - start;
      let timestr = time > 1000 ? (time / 100 | 0) / 10 + "s" : (time | 0) + "ms";
      resolve({
        stdout,
        stderr,
        exitCode: code,
        time,
        timestr
      });
    });
  });
}

var _class;
let Provider = (_class = class Provider {
  constructor(data, env) {
    this.data = data;
    this.remote = env;
  } //TODO env is unused...


  async getEditorConfig() {
    if (this.editorConfig) return this.editorConfig;
    this.editorConfig = await lib.makeAPIRequest({
      env: this.remote,
      path_full: this.data.links.editorConfig
    });
    this.editorConfig.fileExt = await this.getFileExtension();
    return this.editorConfig;
  }

  static async getProviders(env) {
    let providers = await lib.indexPath(env, "/providerTypes?page=1p50");
    providers = providers.sort((a, b) => {
      return a.attributes.category.localeCompare(b.attributes.category) || a.attributes.name.localeCompare(b.attributes.name);
    });
    return new Collection(providers.map(x => new Provider(x, env)));
  }

  async getFileExtension() {
    let config = await this.getEditorConfig();
    let map = {
      python: "py",
      text: "txt",

      getmap(key) {
        if (this.name === "Aurora") return "zip";
        if (this[key]) return this[key];
        return key;
      }

    };
    return map.getmap(config.lang);
  }

  chalkPrint(pad = true) {
    let id = String(this.id);
    if (pad) id = id.padStart(4);
    return chalk`{green ${id}}: {blue ${this.category}} - {green ${this.name}}`;
  }

}, (_applyDecoratedDescriptor(_class, "getProviders", [cached], Object.getOwnPropertyDescriptor(_class, "getProviders"), _class)), _class);
defineAssoc(Provider, "id", "id");
defineAssoc(Provider, "name", "attributes.name");
defineAssoc(Provider, "category", "attributes.category");

var _class$1;
let Preset = (_class$1 = class Preset extends RallyBase {
  constructor({
    path: path$$1,
    remote,
    data
  }) {
    super();
    this.remote = remote;

    if (lib.isLocalEnv(this.remote)) {
      this.path = path$$1;
      let pathspl = this.path.split(".");
      this.ext = pathspl[pathspl.length - 1];

      try {
        this.code = this.getLocalCode();
      } catch (e) {
        log(chalk`{red Node Error} ${e.message}`);
        throw new AbortError("Could not load code of local file");
      }

      let name = this.parseFilenameForName() || this.parseCodeForName();

      try {
        this.data = this.getLocalMetadata();
      } catch (e) {
        this.data = Preset.newShell();
      }

      this.name = name;
      this.isGeneric = true;
    } else {
      this.data = data; //this.name = data.attributes.name;
      //this.id = data.id;

      this.isGeneric = false;
    }
  }

  static newShell() {
    return {
      "attributes": {},
      "relationships": {}
    };
  }

  cleanup() {
    super.cleanup();
    delete this.attributes["createdAt"];
    delete this.attributes["updatedAt"];
  }

  async acclimatize(env) {
    if (!this.isGeneric) throw AbortError("Cannot acclimatize non-generics or shells");
    let providers = await Provider.getProviders(env);
    let ptype = this.relationships["providerType"];
    ptype = ptype.data;
    let provider = providers.findByName(ptype.name);
    ptype.id = provider.id;
  }

  async resolve() {
    if (this.isGeneric) return;
    let providers = await Provider.getProviders(this.remote);
    let proType = this.resolveField(providers, "providerType");
    this.ext = await proType.getFileExtension();
    this.isGeneric = true;
    return {
      proType
    };
  }

  async saveLocal() {
    fs__default.writeFileSync(this.localmetadatapath, JSON.stringify(this.data, null, 4));
    fs__default.writeFileSync(this.localpath, this.code);
  }

  async uploadRemote(env) {
    await this.uploadCodeToEnv(env, true);
  }

  async save(env) {
    if (!this.isGeneric) {
      await this.resolve();
    }

    this.cleanup();

    if (lib.isLocalEnv(env)) {
      await this.saveLocal();
    } else {
      await this.uploadRemote(env);
    }
  }

  async downloadCode() {
    if (this.code) return this.code;
    return this.code = await lib.makeAPIRequest({
      env: this.remote,
      path_full: this.data.links.providerData,
      json: false
    });
  }

  get code() {
    if (this._code) return this._code;
  }

  set code(v) {
    this._code = v;
  }

  chalkPrint(pad = true) {
    let id = String("P-" + (this.remote && this.remote + "-" + this.id) || "Local");
    if (pad) id = id.padStart(10);
    return chalk`{green ${id}}: {blue ${this.name}}`;
  }

  parseFilenameForName() {
    if (this.path.endsWith(".jinja") || this.path.endsWith(".json")) {
      return path.basename(this.path).replace("_", " ").replace("-", " ");
    }
  }

  parseCodeForName() {
    const name_regex = /name:\s([\w\d. \/]+)[\r\s\n]*?/;
    const match = name_regex.exec(this.code);
    if (match) return match[1];
  }

  findStringsInCode(strings) {
    if (!this.code) return [];
    return strings.filter(str => {
      let regex = new RegExp(str);
      return !!this.code.match(regex);
    });
  }

  get localpath() {
    return path__default.join(configObject.repodir, "silo-presets", this.name + "." + this.ext);
  }

  get path() {
    if (this._path) return this._path;
  }

  set path(val) {
    this._path = val;
  }

  get localmetadatapath() {
    return path__default.join(configObject.repodir, "silo-metadata", this.name + ".json");
  }

  get immutable() {
    return this.name.includes("Constant");
  }

  async uploadPresetData(env, id) {
    let res = await lib.makeAPIRequest({
      env,
      path: `/presets/${id}/providerData`,
      body: this.code,
      method: "PUT",
      fullResponse: true
    });
    write(chalk`response {yellow ${res.statusCode}} `);
  }

  async uploadCodeToEnv(env, includeMetadata) {
    write(chalk`Uploading preset {green ${this.name}} to {green ${env}}: `);

    if (this.immutable) {
      log(chalk`{magenta IMMUTABLE}. Nothing to do.`);
      return;
    } //First query the api to see if this already exists.


    let remote = await Preset.getByName(env, this.name);

    if (remote) {
      //If it exists we can replace it
      write("replace, ");

      if (includeMetadata) {
        await lib.makeAPIRequest({
          env,
          path: `/presets/${remote.id}`,
          method: "PATCH",
          payload: {
            data: {
              attributes: this.data.attributes,
              type: "presets"
            }
          }
        });
        write("metadata OK, ");
      }

      await this.uploadPresetData(env, remote.id);
    } else {
      write("create, ");
      let metadata = {
        data: this.data
      };

      if (!this.relationships["providerType"]) {
        throw new AbortError("Cannot acclimatize shelled presets. (try creating it on the env first)");
      }

      await this.acclimatize(env);
      write("Posting to create preset... ");
      let res = await lib.makeAPIRequest({
        env,
        path: `/presets`,
        method: "POST",
        payload: metadata
      });
      let id = res.data.id;
      write(chalk`Created id {green ${id}}... Uploading Code... `);
      await this.uploadPresetData(env, id);
    }

    log("Done");
  }

  constructMetadata(providerID) {
    return {
      attributes: {
        name: this.name //providerSettings: {
        //},

      },
      relationships: {
        providerType: {
          data: {
            id: providerID,
            type: "providerTypes"
          }
        }
      },
      type: "presets"
    };
  }

  getLocalMetadata() {
    return fs__default.readFileSync(this.localmetadatapath, "utf-8");
  }

  getLocalCode() {
    return fs__default.readFileSync(this.path, "utf-8");
  }

  static async getByName(env, name) {
    if (Preset.hasLoadedAll) {
      return (await Preset.getPresets(env)).findByName(name);
    } else {
      let data = await lib.makeAPIRequest({
        env,
        path: "/presets",
        qs: {
          filter: `name=${name}`
        }
      });
      if (data.data[0]) return new Preset({
        data: data.data[0],
        remote: env
      });
    }
  }

  static async getPresets(env) {
    Preset.hasLoadedAll = true;
    let data = await lib.indexPathFast(env, "/presets?page=1p20");
    return new Collection(data.map(dat => new Preset({
      remote: env,
      data: dat
    })));
  }

}, (_applyDecoratedDescriptor(_class$1, "getPresets", [cached], Object.getOwnPropertyDescriptor(_class$1, "getPresets"), _class$1)), _class$1);
defineAssoc(Preset, "name", "attributes.name");
defineAssoc(Preset, "id", "id");
defineAssoc(Preset, "attributes", "attributes");
defineAssoc(Preset, "relationships", "relationships");

var _class$2;
let Notification = (_class$2 = class Notification {
  constructor(data, env) {
    this.data = data;
    this.remote = env;
  }

  static async getNotifications(env) {
    let notifications = await lib.indexPath(env, "/notificationPresets?page=1p25");
    notifications = notifications.sort((a, b) => {
      return a.attributes.type.localeCompare(b.attributes.type) || a.attributes.name.localeCompare(b.attributes.name);
    });
    return new Collection(notifications.map(x => new Notification(x, env)));
  }

  chalkPrint(pad = false) {
    let id = String("N-" + this.id);
    if (pad) id = id.padStart(4);
    return chalk`{green ${id}}: {blue ${this.type}} - {green ${this.name}}`;
  }

}, (_applyDecoratedDescriptor(_class$2, "getNotifications", [cached], Object.getOwnPropertyDescriptor(_class$2, "getNotifications"), _class$2)), _class$2);
defineAssoc(Notification, "id", "id");
defineAssoc(Notification, "name", "attributes.name");
defineAssoc(Notification, "address", "attributes.address");
defineAssoc(Notification, "type", "attributes.type");

var _class$3;
let Rule = (_class$3 = class Rule extends RallyBase {
  constructor(data, remote) {
    super();
    this.data = data;
    this.remote = remote;
    this.isGeneric = !this.remote;
  }

  async acclimatize(env) {
    let presets = await Preset.getPresets(env);
    let rules = await Rule.getRules(env);
    let providers = await Provider.getProviders(env);
    let notifications = await Notification.getNotifications(env);
    let preset = this.resolveField(presets, "preset", false, "specific");
    let pNext = this.resolveField(rules, "passNext", false, "specific");
    let eNext = this.resolveField(rules, "errorNext", false, "specific");
    let proType = this.resolveField(providers, "providerType", false, "specific");
    let enterNotif = this.resolveField(notifications, "enterNotifications", true, "specific");
    let errorNotif = this.resolveField(notifications, "errorNotifications", true, "specific");
    let passNotif = this.resolveField(notifications, "passNotifications", true, "specific");
  }

  async saveA(env) {
    if (lib.isLocalEnv(env)) return;
    return await this.createIfNotExist(env);
  }

  async saveB(env) {
    if (!this.isGeneric) {
      await this.resolve();
    }

    this.cleanup();

    if (lib.isLocalEnv(env)) {
      fs__default.writeFileSync(this.localpath, JSON.stringify(this.data, null, 4));
    } else {
      await this.acclimatize(env);
      await this.uploadRemote(env);
    }
  }

  get immutable() {
    return false;
  }

  async createIfNotExist(env) {
    write(chalk`First pass rule {green ${this.name}} to {green ${env}}: `);

    if (this.immutable) {
      log(chalk`{magenta IMMUTABLE}. Nothing to do.`);
      return;
    } //First query the api to see if this already exists.


    let remote = await Rule.getByName(env, this.name);
    this.idMap = this.idMap || {};

    if (remote) {
      this.idMap[env] = remote.id;
      log(chalk`exists ${remote.chalkPrint(false)}`);
      return;
    } //If it exists we can replace it


    write("create, ");
    let res = await lib.makeAPIRequest({
      env,
      path: `/workflowRules`,
      method: "POST",
      payload: {
        data: {
          attributes: {
            name: this.name
          },
          type: "workflowRules"
        }
      }
    });
    this.idMap = this.idMap || {};
    this.idMap[env] = res.data.id;
    write("id ");
    log(this.idMap[env]);
  }

  async patchStrip() {
    delete this.data.attributes.createdAt;
    delete this.data.attributes.starred;
    delete this.data.attributes.updatedAt;

    for (let key in this.relationships) {
      let relationship = this.relationships[key];

      if (!relationship.data || relationship.data instanceof Array && !relationship.data[0]) {
        delete this.relationships[key];
      }
    }
  }

  async uploadRemote(env) {
    write(chalk`Uploading rules {green ${this.name}} to {green ${env}}: `);

    if (this.immutable) {
      log(chalk`{magenta IMMUTABLE}. Nothing to do.`);
      return;
    }

    if (this.idMap[env]) {
      await this.patchStrip(); //If it exists we can replace it

      write("replace, ");
      let res = await lib.makeAPIRequest({
        env,
        path: `/workflowRules/${this.idMap[env]}`,
        method: "PATCH",
        payload: {
          data: this.data
        },
        fullResponse: true
      });
      log(chalk`response {yellow ${res.statusCode}}`);

      if (res.statusCode !== 200) {
        log(res.body);
        log(JSON.stringify(this.data, null, 4));
      }
    } else {
      throw Error("Bad idmap!");
    }
  }

  get localpath() {
    return path__default.join(configObject.repodir, "silo-rules", this.name + ".json");
  }

  async resolve() {
    let presets = await Preset.getPresets(this.remote);
    let rules = await Rule.getRules(this.remote);
    let providers = await Provider.getProviders(this.remote);
    let notifications = await Notification.getNotifications(this.remote);
    let preset = this.resolveField(presets, "preset", false);
    let pNext = this.resolveField(rules, "passNext", false);
    let eNext = this.resolveField(rules, "errorNext", false);
    let proType = this.resolveField(providers, "providerType", false);
    let enterNotif = this.resolveField(notifications, "enterNotifications", true);
    let errorNotif = this.resolveField(notifications, "errorNotifications", true);
    let passNotif = this.resolveField(notifications, "passNotifications", true); //TODO Unsupported

    delete this.relationships["enterMetadata"];
    delete this.relationships["errorMetadata"];
    delete this.relationships["dynamicNexts"];
    this.isGeneric = true;
    return {
      preset,
      proType,
      pNext,
      eNext,
      errorNotif,
      enterNotif,
      passNotif
    };
  }

  chalkPrint(pad = true) {
    let id = String("R-" + this.remote + "-" + this.id);
    if (pad) id = id.padStart(10);
    return chalk`{green ${id}}: {blue ${this.name}}`;
  }

  static async getByName(env, name) {
    return (await Rule.getRules(env)).findByName(name);
  }

  static async getRules(env) {
    let rules = await lib.indexPathFast(env, "/workflowRules?page=1p20");
    return new Collection(rules.map(data => new Rule(data, env)));
  }

}, (_applyDecoratedDescriptor(_class$3, "getRules", [cached], Object.getOwnPropertyDescriptor(_class$3, "getRules"), _class$3)), _class$3);
defineAssoc(Rule, "name", "attributes.name");
defineAssoc(Rule, "id", "id");
defineAssoc(Rule, "relationships", "relationships");

class SupplyChain {
  constructor(startingRule, stopRule) {
    this.startingRule = startingRule;
    this.stopRule = stopRule;
    this.remote = startingRule.remote;
  }

  async calculate() {
    write("Getting rules... ");
    this.allRules = await Rule.getRules(this.remote);
    log(this.allRules.length);
    write("Getting presets... ");
    this.allPresets = await Preset.getPresets(this.remote);
    log(this.allPresets.length);
    write("Getting providers... ");
    this.allProviders = await Provider.getProviders(this.remote);
    log(this.allProviders.length);
    write("Getting notifications... ");
    this.allNotifications = await Notification.getNotifications(this.remote);
    log(this.allNotifications.length);
    write("Downloading code... ");
    await Promise.all(this.allPresets.arr.map(obj => obj.downloadCode()));
    log("Done!"); //fs.writeFileSync("test.json", JSON.stringify(this, null, 4))
    //Now we have everything we need to find a whole supply chain

    write("Calculating Supply chain... ");
    log(this.startingRule.chalkPrint());
    let allRuleNames = this.allRules.arr.map(x => x.name).filter(x => x.length >= 4);
    let allPresetNames = this.allPresets.arr.map(x => x.name).filter(x => x.length >= 4);
    let allNotifNames = this.allNotifications.arr.map(x => x.name).filter(x => x.length >= 4);
    let requiredNotifications = new Set();
    let ruleQueue = [this.startingRule];
    let presetQueue = [];

    for (let currentRule of ruleQueue) {
      if (currentRule === this.stopRule) continue;
      let {
        eNext,
        pNext,
        preset,
        passNotif,
        errorNotif,
        enterNotif
      } = await currentRule.resolve();
      passNotif.forEach(n => requiredNotifications.add(n));
      enterNotif.forEach(n => requiredNotifications.add(n));
      errorNotif.forEach(n => requiredNotifications.add(n));
      if (eNext && !ruleQueue.includes(eNext)) ruleQueue.push(eNext);
      if (pNext && !ruleQueue.includes(eNext)) ruleQueue.push(pNext);
      let neededPresets = preset.findStringsInCode(allPresetNames);
      neededPresets = neededPresets.map(x => this.allPresets.findByName(x));
      let neededRules = preset.findStringsInCode(allRuleNames);
      neededRules = neededRules.map(x => this.allRules.findByName(x));
      preset.findStringsInCode(allNotifNames).map(str => this.allNotifications.findByName(str)).forEach(notif => requiredNotifications.add(notif));

      for (let p of neededPresets) if (!presetQueue.includes(p)) presetQueue.push(p);

      for (let p of neededRules) if (!ruleQueue.includes(p)) ruleQueue.push(p);

      if (configObject.verbose) {
        write(currentRule.chalkPrint(false));
        log(":");
        write("  ");
        write(preset.chalkPrint(false));
        log(":");
        write("  Pass Next: ");
        if (pNext) write(pNext.chalkPrint(false));else write("None");
        log("");
        write("  Err  Next: ");
        if (eNext) write(eNext.chalkPrint(false));else write("None");
        log("");
        log("  Rules:");

        for (let p of neededRules) log("    " + p.chalkPrint(true));

        log("  Presets:");

        for (let p of neededPresets) log("    " + p.chalkPrint(true));

        log("\n");
      }
    }

    log("Done!");
    this.rules = new Collection(ruleQueue);
    this.presets = new Collection(presetQueue);
    requiredNotifications.delete(undefined);
    this.notifications = new Collection([...requiredNotifications]);
  }

  async log() {
    log("Required notifications: ");
    this.notifications.log();
    write("Required rules: ");
    log(this.rules.arr.length);
    this.rules.log();
    write("Required presets: ");
    log(this.presets.arr.length);
    this.presets.log();
  }

  async syncTo(env) {
    for (let preset of this.presets) {
      await preset.save(env);
    }

    for (let rule of this.rules) {
      await rule.saveA(env);
    }

    log("");
    log("OK");
    log("Uncaching UAT");
    Rule.getRules.remove([env]);

    for (let rule of this.rules) {
      await rule.saveB(env);
    }
  }

}

require("source-map-support").install();
const rallyFunctions = {
  async bestPagintation() {
    global.silentAPI = true;

    for (let i = 10; i <= 30; i += 5) {
      console.time("test with " + i);
      let dl = await lib.indexPathFast("DEV", `/workflowRules?page=1p${i}`);
      console.timeEnd("test with " + i);
    }
  },

  async uploadPresets(env, presets, createFunc = () => false) {
    for (let preset of presets) {
      await preset.uploadCodeToEnv(env, createFunc);
    }
  },

  //Dummy test access
  async testAccess(env) {
    if (lib.isLocalEnv(env)) {
      //TODO
      return true;
    }

    let result = await lib.makeAPIRequest({
      env,
      path: "/providers?page=1p1",
      fullResponse: true
    });
    return result.statusCode;
  }

};

var allIndexBundle = /*#__PURE__*/Object.freeze({
  rallyFunctions: rallyFunctions,
  SupplyChain: SupplyChain,
  Preset: Preset,
  Rule: Rule,
  Provider: Provider,
  Notification: Notification,
  configFile: configFile,
  get configObject () { return configObject; },
  lib: lib,
  AbortError: AbortError,
  APIError: APIError,
  UnconfiguredEnvError: UnconfiguredEnvError,
  ProtectedEnvError: ProtectedEnvError,
  Collection: Collection,
  RallyBase: RallyBase
});

var version = "1.6.3";

const inquirer = importLazy("inquirer");
async function $api(propArray) {
  const defaults = {
    DEV: "https://discovery-dev.sdvi.com/api/v2",
    UAT: "https://discovery-uat.sdvi.com/api/v2",
    PROD: "https://discovery.sdvi.com/api/v2"
  };
  let q;

  if (propArray && propArray[1]) {
    q = {
      envs: [propArray[1]]
    };
  } else {
    //Create a checkbox prompt to choose enviornments
    q = await inquirer.prompt([{
      type: "checkbox",
      name: "envs",
      message: `What enviornments would you like to configure?`,
      choices: Object.keys(defaults).map(name => ({
        name,
        checked: true
      }))
    }]);
  } //Each env should ask 2 for two things: The url and the key.


  let questions = q.envs.map(env => {
    let defaultKey = process.env[`rally_api_key_${env}`];

    if (configObject && configObject.api && configObject.api[env]) {
      defaultKey = configObject.api[env].key;
    }

    return [{
      type: "input",
      name: `api.${env}.url`,
      message: `What is the url endpoint for ${env}`,
      default: defaults[env]
    }, {
      type: "input",
      name: `api.${env}.key`,
      message: `What is your api key for ${env}`,
      default: defaultKey
    }];
  }); //flatten and ask

  questions = [].concat(...questions);
  q = await inquirer.prompt(questions);

  if (propArray) {
    q.api = { ...configObject.api,
      ...q.api
    };
  }

  return q;
}
async function $chalk(propArray) {
  return {
    chalk: await askQuestion("Would you like chalk enabled (Adds coloring)?")
  };
}
async function $restrictUAT(propArray) {
  return {
    restrictUAT: await askQuestion("Would you like to protect UAT?")
  };
}
async function $repodir(propArray) {
  return await inquirer.prompt([{
    type: "input",
    name: `repodir`,
    message: `Where is your rally repository?`,
    default: process.env["rally_repo_path"]
  }]);
}
async function $defaultEnv(propArray) {
  return await inquirer.prompt([{
    type: "input",
    name: `defaultEnv`,
    message: `Default enviornment?`,
    default: "DEV"
  }]);
} //Internal usage/testing

async function selectProvider(env, providers) {
  let defaultProvider = providers.find(x => x.attributes.name === "SdviEvaluate");

  if (args.defaultSelect) {
    return defaultProvider;
  } else {
    let q = await inquirer.prompt([{
      type: "list",
      name: "provider",
      default: defaultProvider,
      choices: providers.map(x => ({
        name: prettyPrintProvider(x),
        value: x
      }))
    }]);
    return q.provider;
  }
}
async function askQuestion(question) {
  return (await inquirer.prompt([{
    type: "confirm",
    name: "ok",
    message: question
  }])).ok;
}

var configHelpers = /*#__PURE__*/Object.freeze({
  $api: $api,
  $chalk: $chalk,
  $restrictUAT: $restrictUAT,
  $repodir: $repodir,
  $defaultEnv: $defaultEnv,
  selectProvider: selectProvider,
  askQuestion: askQuestion
});

var _dec, _dec2, _dec3, _dec4, _dec5, _dec6, _dec7, _dec8, _dec9, _dec10, _dec11, _dec12, _dec13, _dec14, _dec15, _dec16, _dec17, _dec18, _dec19, _dec20, _dec21, _dec22, _dec23, _dec24, _dec25, _dec26, _dec27, _dec28, _dec29, _dec30, _dec31, _obj;

require("source-map-support").install();
let argv = argparse(process.argv.slice(2), {
  string: ["file", "env"],
  //boolean: ["no-protect"],
  default: {
    protect: true
  },
  alias: {
    f: "file",
    e: "env"
  }
}); //help menu helper

function printHelp(help, short) {
  let helpText$$1 = chalk`
{white ${help.name}}: ${help.text}
    Usage: ${help.usage || "<unknown>"}
`; //Trim newlines

  helpText$$1 = helpText$$1.substring(1, helpText$$1.length - 1);

  if (!short) {
    for (let param$$1 of help.params || []) {
      helpText$$1 += chalk`\n    {blue ${param$$1.param}}: ${param$$1.desc}`;
    }

    for (let arg$$1 of help.args || []) {
      helpText$$1 += chalk`\n    {blue ${arg$$1.short}}, {blue ${arg$$1.long}}: ${arg$$1.desc}`;
    }
  }

  return helpText$$1;
}

let presetsub = {
  async before(args) {
    this.env = args.env;
    if (!this.env) throw new AbortError("No env supplied");

    if (args.file) {
      let files = args.file;
      if (typeof files === "string") files = [files];
      this.files = files;
    } else if (args._.shift() == "-") {
      log("Reading from stdin");

      let getStdin = require("get-stdin");

      let stdin = await getStdin();
      this.files = stdin.split("\n");
      if (this.files[this.files.length - 1] === "") this.files.pop();
    }
  },

  async $list(args) {
    log("Loading...");
    let presets = await Preset.getPresets(this.env);
    if (configObject.rawOutput) return presets;
    log(chalk`{yellow ${presets.length}} presets on {green ${this.env}}.`);

    for (let preset of presets) log(preset.chalkPrint());
  },

  async $upload(args) {
    if (!this.files) {
      throw new AbortError("No files provided to upload (use --file argument)");
    }

    log(chalk`Uploading {green ${this.files.length}} preset(s) to {green ${this.env}}.`);
    let presets = this.files.map(path$$1 => new Preset({
      path: path$$1,
      remote: false
    }));
    await rallyFunctions.uploadPresets(this.env, presets);
  },

  async $diff(args) {
    let file = this.files[0];

    if (!this.files) {
      throw new AbortError("No files provided to diff (use --file argument)");
    }

    let preset = new Preset({
      path: file,
      remote: false
    });

    if (!preset.name) {
      throw new AbortError(chalk`No preset header found. Cannot get name.`);
    }

    let preset2 = await Preset.getByName(this.env, preset.name);

    if (!preset2) {
      throw new AbortError(chalk`No preset found with name {red ${preset.name}} on {blue ${this.env}}`);
    }

    await preset2.downloadCode();

    let tempfile = require("tempy").file;

    let temp = tempfile({
      extension: preset.ext
    });
    fs.writeFileSync(temp, preset2.code);
    let ptr = `${file},${temp}`; //raw output returns "file1" "file2"

    if (configObject.rawOutput) return ptr; //standard diff

    argv.command = argv.command || "diff";
    await spawn(argv.command, [file, temp], {
      stdio: "inherit"
    });
  },

  async unknown(arg$$1, args) {
    log(chalk`Unknown action {red ${arg$$1}} try '{white rally help preset}'`);
  }

};
let rulesub = {
  async before(args) {
    this.env = args.env;
    if (!this.env) throw new AbortError("No env supplied");
  },

  async $list(args) {
    log("Loading...");
    let rules = await Rule.getRules(this.env);
    if (configObject.rawOutput) return rules;
    log(chalk`{yellow ${rules.length}} rules on {green ${this.env}}.`);

    for (let rule of rules) log(rule.chalkPrint());
  },

  async unknown(arg$$1, args) {
    log(chalk`Unknown action {red ${arg$$1}} try '{white rally help rule}'`);
  }

};
let jupytersub = {
  async before(args) {
    this.input = args._.shift() || "main.ipynb";
    this.output = args._.shift() || "main.py";
  },

  async $build(args) {
    let cmd = `jupyter nbconvert --to python ${this.input} --TagRemovePreprocessor.remove_cell_tags={\"remove_cell\"} --output ${this.output} --TemplateExporter.exclude_markdown=True --TemplateExporter.exclude_input_prompt=True --TemplateExporter.exclude_output_prompt=True`.split(" ");
    log(chalk`Compiling GCR file {green ${this.input}} into {green ${this.output}} using jupyter...`);

    try {
      let {
        timestr
      } = await spawn(cmd[0], cmd.slice(1));
      log(chalk`Complete in ~{green.bold ${timestr}}.`);
    } catch (e) {
      if (e.code !== "ENOENT") throw e;
      log(chalk`Cannot run the build command. Make sure that you have jupyter notebook installed.\n{green pip install jupyter}`);
      return;
    }
  }

};
let supplysub = {
  async before(args) {
    this.env = args.env;
    if (!this.env) throw new AbortError("No env supplied");
  },

  //Calculate a supply chain based on a starting rule at the top of the stack
  async $calc(args) {
    let name$$1 = args._.shift();

    if (!name$$1) {
      throw new AbortError("No starting rule supplied");
    }

    let rules = await Rule.getRules(this.env);
    let start = rules.findByNameContains(name$$1);

    if (!start) {
      throw new AbortError(chalk`No starting rule found by name {blue ${name$$1}}`);
    }

    log(chalk`Analzying supply chain: ${start.chalkPrint(false)}`);
    let chain = new SupplyChain(start);
    await chain.calculate();

    if (args["to"]) {
      await chain.syncTo(args["to"]);
    } else {
      await chain.log();
    }
  },

  async $magic(args) {
    let big = require("fs").readFileSync("test.json");

    big = JSON.parse(big);
    let presets = big.allPresets.arr.map(obj => {
      let preset = new Preset({
        data: obj.data,
        remote: big.remote
      });
      preset.code = obj._code;
      return preset;
    });
    Preset.getPresets.cachePush([big.remote], new Collection(presets));
    let rules = big.allRules.arr.map(obj => {
      let rule = new Rule(obj.data, big.remote);
      return rule;
    });
    Rule.getRules.cachePush([big.remote], new Collection(rules));
    return await this.$calc(args);
  },

  async unknown(arg$$1, args) {
    log(chalk`Unknown action {red ${arg$$1}} try '{white rally help supply}'`);
  }

};

function subCommand(object) {
  object = {
    before() {},

    after() {},

    unknown() {},

    ...object
  };
  return async function (args) {
    //Grab the next arg on the stack, find a function tied to it, and run
    let arg$$1 = args._.shift();

    let key = "$" + arg$$1;
    let ret;

    if (object[key]) {
      await object.before(args);
      ret = await object[key](args);
      await object.after(args);
    } else {
      if (arg$$1 === undefined) arg$$1 = "(None)";
      object.unknown(arg$$1, args);
    }

    return ret;
  };
}

let cli = (_dec = helpText(`Display the help menu`), _dec2 = usage(`rally help [subhelp]`), _dec3 = param("subhelp", "The name of the command to see help for"), _dec4 = helpText("Rally tools jupyter interface. Requires jupyter to be installed."), _dec5 = usage("rally jupyter build [in] [out]"), _dec6 = param("in/out", "input and output file for jupyter. By default main.ipyrb and main.py"), _dec7 = helpText(`Preset related actions`), _dec8 = usage(`rally preset [action] --env <enviornment> --file [file1] --file [file2] ...`), _dec9 = param("action", "The action to perform. Can be upload, diff, list"), _dec10 = arg("-e", "--env", "The enviornment you wish to perform the action on"), _dec11 = arg("-f", "--file", "A file to act on"), _dec12 = arg("~", "--command", "If the action is diff, this is the command to run instead of diff"), _dec13 = helpText(`Rule related actions`), _dec14 = usage(`rally rule [action] --env [enviornment]`), _dec15 = param("action", "The action to perform. Only list is supported right now"), _dec16 = arg("-e", "--env", "The enviornment you wish to perform the action on"), _dec17 = helpText(`supply chain related actions`), _dec18 = usage(`rally supply [action] [identifier] --env [enviornment]`), _dec19 = param("action", "The action to perform. Can be calc."), _dec20 = param("identifier", "If the action is calc, then this identifier should be the first rule in the chain."), _dec21 = arg("-e", "--env", "The enviornment you wish to perform the action on"), _dec22 = helpText(`List all available providers, or find one by name/id`), _dec23 = usage(`rally providers [identifier] --env [env] --raw`), _dec24 = param("identifier", "Either the name or id of the provider"), _dec25 = arg("-e", "--env", "The enviornment you wish to perform the action on"), _dec26 = arg("~", "--raw", "Raw output of command. If [identifier] is given, then print editorConfig too"), _dec27 = helpText(`Change config for rally tools`), _dec28 = usage("rally config [key] --set [value] --raw"), _dec29 = param("key", chalk`Key you want to edit. For example, {green chalk} or {green api.DEV}`), _dec30 = arg("~", "--set", "If this value is given, no interactive prompt will launch and the config option will change."), _dec31 = arg("~", "--raw", "Raw output of json config"), (_obj = {
  async help(args) {
    let arg$$1 = args._.shift();

    if (arg$$1) {
      let help = helpEntries[arg$$1];

      if (!help) {
        log(chalk`No help found for '{red ${arg$$1}}'`);
      } else {
        log(printHelp(helpEntries[arg$$1]));
      }
    } else {
      for (let helpArg in helpEntries) {
        log(printHelp(helpEntries[helpArg], true));
      }
    }
  },

  async jupyter(args) {
    return subCommand(jupytersub)(args);
  },

  //@helpText(`Print input args, for debugging`)
  async printArgs(args) {
    log(args);
  },

  async preset(args) {
    return subCommand(presetsub)(args);
  },

  async rule(args) {
    return subCommand(rulesub)(args);
  },

  async supply(args) {
    return subCommand(supplysub)(args);
  },

  async providers(args) {
    let env = args.env;
    if (!env) return errorLog("No env supplied.");

    let ident = args._.shift();

    let providers = await Provider.getProviders(env);

    if (ident) {
      let pro = providers.arr.find(x => x.id == ident || x.name.includes(ident));

      if (!pro) {
        log(chalk`Couldn't find provider by {green ${ident}}`);
      } else {
        log(pro.chalkPrint(false));
        let econfig = await pro.getEditorConfig();

        if (args.raw) {
          return pro;
        } else {
          if (econfig.helpText.length > 100) {
            econfig.helpText = "<too long to display>";
          }

          if (econfig.completions.length > 5) {
            econfig.completions = "<too long to display>";
          }

          log(econfig);
        }
      }
    } else {
      if (args.raw) return providers;

      for (let pro of providers) log(pro.chalkPrint());
    }
  },

  async config(args) {
    let prop = args._.shift();

    let propArray = prop && prop.split("."); //if(!await configHelpers.askQuestion(`Would you like to create a new config file in ${configFile}`)) return;

    let newConfigObject;

    if (!prop) {
      if (configObject.rawOutput) return configObject;
      log("Creating new config");
      newConfigObject = { ...configObject
      };

      for (let helperName in configHelpers) {
        if (helperName.startsWith("$")) {
          newConfigObject = { ...newConfigObject,
            ...(await configHelpers[helperName](false))
          };
        }
      }
    } else {
      log(chalk`Editing option {green ${prop}}`);

      if (args.set) {
        newConfigObject = { ...configObject,
          [prop]: args.set
        };
      } else {
        let ident = "$" + propArray[0];

        if (configHelpers[ident]) {
          newConfigObject = { ...configObject,
            ...(await configHelpers[ident](propArray))
          };
        } else {
          log(chalk`No helper for {red ${ident}}`);
          return;
        }
      }
    }

    newConfigObject.hasConfig = true; //Create readable json and make sure the user is ok with it

    let newConfig = JSON.stringify(newConfigObject, null, 4);
    log(newConfig); //-y or --set will make this not prompt

    if (!args.y && !args.set && !(await askQuestion("Write this config to disk?"))) return;
    fs.writeFileSync(configFile, newConfig, {
      mode: 0o600
    });
    log(chalk`Created file {green ${configFile}}.`);
  },

  //Used to test startup and teardown speed.
  noop() {
    return true;
  }

}, (_applyDecoratedDescriptor(_obj, "help", [_dec, _dec2, _dec3], Object.getOwnPropertyDescriptor(_obj, "help"), _obj), _applyDecoratedDescriptor(_obj, "jupyter", [_dec4, _dec5, _dec6], Object.getOwnPropertyDescriptor(_obj, "jupyter"), _obj), _applyDecoratedDescriptor(_obj, "preset", [_dec7, _dec8, _dec9, _dec10, _dec11, _dec12], Object.getOwnPropertyDescriptor(_obj, "preset"), _obj), _applyDecoratedDescriptor(_obj, "rule", [_dec13, _dec14, _dec15, _dec16], Object.getOwnPropertyDescriptor(_obj, "rule"), _obj), _applyDecoratedDescriptor(_obj, "supply", [_dec17, _dec18, _dec19, _dec20, _dec21], Object.getOwnPropertyDescriptor(_obj, "supply"), _obj), _applyDecoratedDescriptor(_obj, "providers", [_dec22, _dec23, _dec24, _dec25, _dec26], Object.getOwnPropertyDescriptor(_obj, "providers"), _obj), _applyDecoratedDescriptor(_obj, "config", [_dec27, _dec28, _dec29, _dec30, _dec31], Object.getOwnPropertyDescriptor(_obj, "config"), _obj)), _obj));

async function unknownCommand(cmd) {
  log(chalk`Unknown command {red ${cmd}}.`);
}

async function noCommand() {
  write(chalk`
Rally Tools {yellow v${version} (alpha)} CLI
by John Schmidt <John_Schmidt@discovery.com>
`); //Prompt users to setup one time config.

  if (!configObject.hasConfig) {
    write(chalk`
It looks like you haven't setup the config yet. Please run '{green rally config}'.
`);
    return;
  } //API Access tests


  for (let env of ["UAT", "DEV", "PROD", "LOCAL"]) {
    //Test access. Returns HTTP response code
    let resultStr;

    try {
      let result = await rallyFunctions.testAccess(env); //Create a colored display and response

      resultStr = chalk`{yellow ${result} <unknown>}`;
      if (result === 200) resultStr = chalk`{green 200 OK}`;else if (result === 401) resultStr = chalk`{red 401 No Access}`;else if (result >= 500) resultStr = chalk`{yellow ${result} API Down?}`;else if (result === true) resultStr = chalk`{green OK}`;else if (result === false) resultStr = chalk`{red BAD}`;
    } catch (e) {
      if (!e instanceof UnconfiguredEnvError) throw e;
      resultStr = chalk`{yellow Unconfigured}`;
    }

    log(chalk`   ${env}: ${resultStr}`);
  }
}

async function $main() {
  // First we need to decide if the user wants color or not. If they do want
  // color, we need to make sure we use the right mode
  chalk.enabled = configObject.hasConfig ? configObject.chalk : true;

  if (chalk.level === 0 || !chalk.enabled) {
    let force = argv["force-color"];

    if (force) {
      chalk.enabled = true;

      if (force === true && chalk.level === 0) {
        chalk.level = 1;
      } else if (Number(force)) {
        chalk.level = Number(force);
      }
    }
  } //This flag being true allows you to modify UAT and PROD


  if (!argv["protect"]) {
    configObject.dangerModify = true;
  } //This enables raw output for some functions


  if (argv["raw"]) {
    configObject.rawOutput = true;

    global.log = () => {};

    global.errorLog = () => {};

    global.write = () => {};
  } //Default enviornment should normally be from config, but it can be
  //overridden by the -e/--env flag


  if (configObject.defaultEnv) {
    argv.env = argv.env || configObject.defaultEnv;
  } //Enable verbose logging in some places.


  if (argv["verbose"]) {
    configObject.verbose = argv["verbose"];
  } //copy argument array to new object to allow modification


  argv._old = argv._.slice(); //Take first argument after `node bundle.js`
  //If there is no argument, display the default version info and API access.

  let func = argv._.shift();

  if (func) {
    if (!cli[func]) return await unknownCommand(func);

    try {
      //Call the cli function
      let ret = await cli[func](argv);

      if (ret) {
        write(chalk.white("CLI returned: "));
        if (ret instanceof Collection) ret = ret.arr; //Directly use console.log so that --raw works as intended.

        if (typeof ret === "object") {
          console.log(JSON.stringify(ret, null, 4));
        } else {
          console.log(ret);
        }
      }
    } catch (e) {
      if (e instanceof AbortError) {
        log(chalk`{red CLI Aborted}: ${e.message}`);
      } else {
        throw e;
      }
    }
  } else {
    await noCommand();
  }
}

async function main$1(...args) {
  //Catch all for errors to avoid ugly default node promise catcher
  try {
    await $main(...args);
  } catch (e) {
    errorLog(e.stack);
  }
} // If this is an imported module, then we should exec the cli interface.
// Oterwise just export everything.


if (require.main === module) {
  main$1();
} else {
  module.exports = allIndexBundle;
}
//# sourceMappingURL=bundle.js.map
